@@ -23,12 +23,17 @@
  */
 
 #include <libwps/libwps.h>
+
+#include "libwps_internal.h"
+#include "libwps_tools_win.h"
+
+#include "Lotus.h"
+#include "Quattro.h"
 #include "WKS4.h"
 #include "WPS4.h"
 #include "WPS8.h"
 #include "WPSHeader.h"
 #include "WPSParser.h"
-#include "libwps_internal.h"
 
 using namespace libwps;
 
@@ -48,16 +53,13 @@ the full 100%.
 
  \warning When compiled with -DDEBUG_WITH__FILES, code is added to store the results of the parsing in different files: one file by Ole parts and some files to store the read pictures. These files are created in the current repository, therefore it is recommended to launch the tests in an empty repository...*/
 
-/**
-Analyzes the content of an input stream to see if it can be parsed
-\param ip The input stream
-\param kind The document kind
-\return A confidence value which represents the likelyhood that the content from
-the input stream can be parsed
-*/
-WPSLIB WPSConfidence WPSDocument::isFileFormatSupported(librevenge::RVNGInputStream *ip, WPSKind &kind)
+WPSLIB WPSConfidence WPSDocument::isFileFormatSupported(librevenge::RVNGInputStream *ip, WPSKind &kind, WPSCreator &creator, bool &needEncoding)
 {
 	WPS_DEBUG_MSG(("WPSDocument::isFileFormatSupported()\n"));
+
+	if (!ip)
+		return WPS_CONFIDENCE_NONE;
+
 	kind=WPS_TEXT;
 	WPSHeaderPtr header;
 	shared_ptr<librevenge::RVNGInputStream > input(ip, WPS_shared_ptr_noop_deleter<librevenge::RVNGInputStream>());
@@ -67,6 +69,7 @@ WPSLIB WPSConfidence WPSDocument::isFileFormatSupported(librevenge::RVNGInputStr
 
 		if (!header)
 			return WPS_CONFIDENCE_NONE;
+		creator = header->getCreator();
 		kind = header->getKind();
 
 		WPSConfidence confidence = WPS_CONFIDENCE_NONE;
@@ -76,6 +79,25 @@ WPSLIB WPSConfidence WPSDocument::isFileFormatSupported(librevenge::RVNGInputStr
 			WPS4Parser parser(header->getInput(), header);
 			if (!parser.checkHeader(header.get(), true))
 				return WPS_CONFIDENCE_NONE;
+			needEncoding=header->getNeedEncoding();
+			return WPS_CONFIDENCE_EXCELLENT;
+		}
+		else if (kind==WPS_SPREADSHEET && creator==WPS_LOTUS && header->getMajorVersion()>=100)
+		{
+			// create a Lotus to check the header validity
+			LotusParser parser(header->getInput(), header);
+			if (!parser.checkHeader(header.get(), true))
+				return WPS_CONFIDENCE_NONE;
+			needEncoding=header->getNeedEncoding();
+			return WPS_CONFIDENCE_EXCELLENT;
+		}
+		else if (kind==WPS_SPREADSHEET && creator==WPS_QUATTRO_PRO && header->getMajorVersion()<=2)
+		{
+			// create a Quattro to check the header validity
+			QuattroParser parser(header->getInput(), header);
+			if (!parser.checkHeader(header.get(), true))
+				return WPS_CONFIDENCE_NONE;
+			needEncoding=header->getNeedEncoding();
 			return WPS_CONFIDENCE_EXCELLENT;
 		}
 		else if (kind==WPS_SPREADSHEET || kind==WPS_DATABASE)
@@ -84,6 +106,7 @@ WPSLIB WPSConfidence WPSDocument::isFileFormatSupported(librevenge::RVNGInputStr
 			WKS4Parser parser(header->getInput(), header);
 			if (!parser.checkHeader(header.get(), true))
 				return WPS_CONFIDENCE_NONE;
+			needEncoding=header->getNeedEncoding();
 			return WPS_CONFIDENCE_EXCELLENT;
 		}
 
@@ -119,15 +142,12 @@ WPSLIB WPSConfidence WPSDocument::isFileFormatSupported(librevenge::RVNGInputStr
 	return WPS_CONFIDENCE_NONE;
 }
 
-/**
-Parses the input stream content. It will make callbacks to the functions provided by a
-librevenge::RVNGTextInterface class implementation when needed. This is often commonly called the
-'main parsing routine'.
-\param ip The input stream
-\param documentInterface A WPSListener implementation
-*/
-WPSLIB WPSResult WPSDocument::parse(librevenge::RVNGInputStream *ip, librevenge::RVNGTextInterface *documentInterface)
+WPSLIB WPSResult WPSDocument::parse(librevenge::RVNGInputStream *ip, librevenge::RVNGTextInterface *documentInterface,
+                                    char const */*password*/, char const *encoding)
 {
+	if (!ip || !documentInterface)
+		return WPS_UNKNOWN_ERROR;
+
 	WPSResult error = WPS_OK;
 
 	WPSHeaderPtr header;
@@ -158,7 +178,8 @@ WPSLIB WPSResult WPSDocument::parse(librevenge::RVNGInputStream *ip, librevenge:
 		case 2:
 		case 1:
 		{
-			parser.reset(new WPS4Parser(header->getInput(), header));
+			parser.reset(new WPS4Parser(header->getInput(), header,
+			                            libwps_tools_win::Font::getTypeForString(encoding)));
 			if (!parser) return WPS_UNKNOWN_ERROR;
 			parser->parse(documentInterface);
 			break;
@@ -187,15 +208,12 @@ WPSLIB WPSResult WPSDocument::parse(librevenge::RVNGInputStream *ip, librevenge:
 	return error;
 }
 
-/**
-Parses the input stream content. It will make callbacks to the functions provided by a
-librevenge::RVNGSpreadsheetInterface class implementation when needed. This is often commonly called the
-'main parsing routine'.
-\param ip The input stream
-\param documentInterface A SpreadsheetInterface implementation
-*/
-WPSLIB WPSResult WPSDocument::parse(librevenge::RVNGInputStream *ip, librevenge::RVNGSpreadsheetInterface *documentInterface)
+WPSLIB WPSResult WPSDocument::parse(librevenge::RVNGInputStream *ip, librevenge::RVNGSpreadsheetInterface *documentInterface,
+                                    char const */*password*/, char const *encoding)
 {
+	if (!ip || !documentInterface)
+		return WPS_UNKNOWN_ERROR;
+
 	WPSResult error = WPS_OK;
 
 	WPSHeaderPtr header;
@@ -208,20 +226,42 @@ WPSLIB WPSResult WPSDocument::parse(librevenge::RVNGInputStream *ip, librevenge:
 		if (!header || (header->getKind() != WPS_SPREADSHEET && header->getKind() != WPS_DATABASE))
 			return WPS_UNKNOWN_ERROR;
 
-		switch (header->getMajorVersion())
+		if (header->getKind() == WPS_SPREADSHEET && header->getCreator() == WPS_LOTUS &&
+		        header->getMajorVersion()>=100)
 		{
-		case 4:
-		case 3:
-		case 2:
-		case 1:
-		{
-			parser.reset(new WKS4Parser(header->getInput(), header));
+			parser.reset(new LotusParser(header->getInput(), header,
+			                             libwps_tools_win::Font::getTypeForString(encoding)));
 			if (!parser) return WPS_UNKNOWN_ERROR;
 			parser->parse(documentInterface);
-			break;
 		}
-		default:
-			break;
+		else if (header->getKind() == WPS_SPREADSHEET && header->getCreator() == WPS_QUATTRO_PRO &&
+		         header->getMajorVersion()<=2)
+		{
+			parser.reset(new QuattroParser(header->getInput(), header,
+			                               libwps_tools_win::Font::getTypeForString(encoding)));
+			if (!parser) return WPS_UNKNOWN_ERROR;
+			parser->parse(documentInterface);
+		}
+		else
+		{
+			switch (header->getMajorVersion())
+			{
+			case 4:
+			case 3:
+			case 2:
+			case 1:
+			{
+				parser.reset(new WKS4Parser(header->getInput(), header,
+				                            libwps_tools_win::Font::getTypeForString(encoding)));
+				if (!parser) return WPS_UNKNOWN_ERROR;
+				parser->parse(documentInterface);
+				break;
+			}
+			default:
+				WPS_DEBUG_MSG(("WPSDocument::parse: find unknown version number\n"));
+				error=WPS_UNKNOWN_ERROR;
+				break;
+			}
 		}
 	}
 	catch (libwps::FileException)
